@@ -1,21 +1,8 @@
 /*
-*   Copyright (C) 2017 by Marco Martin <mart@kde.org>
-*
-*   This program is free software; you can redistribute it and/or modify
-*   it under the terms of the GNU Library General Public License as
-*   published by the Free Software Foundation; either version 2, or
-*   (at your option) any later version.
-*
-*   This program is distributed in the hope that it will be useful,
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-*   GNU Library General Public License for more details
-*
-*   You should have received a copy of the GNU Library General Public
-*   License along with this program; if not, write to the
-*   Free Software Foundation, Inc.,
-*   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
-*/
+ *  SPDX-FileCopyrightText: 2017 by Marco Martin <mart@kde.org>
+ *
+ *  SPDX-License-Identifier: LGPL-2.0-or-later
+ */
 
 #include "platformtheme.h"
 #include "kirigamipluginfactory.h"
@@ -63,6 +50,12 @@ public:
     QColor backgroundColor;
     QColor alternateBackgroundColor;
     QColor highlightColor;
+    QColor activeBackgroundColor;
+    QColor linkBackgroundColor;
+    QColor visitedLinkBackgroundColor;
+    QColor negativeBackgroundColor;
+    QColor neutralBackgroundColor;
+    QColor positiveBackgroundColor;
 
     QColor focusColor;
     QColor hoverColor;
@@ -83,6 +76,12 @@ public:
     QColor customBackgroundColor;
     QColor customAlternateBackgroundColor;
     QColor customHighlightColor;
+    QColor customActiveBackgroundColor;
+    QColor customLinkBackgroundColor;
+    QColor customVisitedLinkBackgroundColor;
+    QColor customNegativeBackgroundColor;
+    QColor customNeutralBackgroundColor;
+    QColor customPositiveBackgroundColor;
 
     QColor customFocusColor;
     QColor customHoverColor;
@@ -90,6 +89,8 @@ public:
     QPalette customPalette;
 
     QFont font;
+    QFont smallFont;
+
     bool m_inherit = true;
     bool m_init = true;
     bool m_supportsIconColoring = false;
@@ -108,30 +109,34 @@ PlatformThemePrivate::PlatformThemePrivate(PlatformTheme *q)
 PlatformThemePrivate::~PlatformThemePrivate()
 {}
 
-void setPaletteColor(QPalette& customPalette, QPalette::ColorGroup cg, QPalette::ColorRole cr, const QColor &color)
+void setPaletteColor(QPalette& customPalette, QPalette::ColorGroup cg, QPalette::ColorRole cr, const QColor &color, bool *changed)
 {
     if (customPalette.color(cg, cr) != color) {
+        *changed = true;
         customPalette.setColor(cg, cr, color);
     }
 }
 
 void PlatformThemePrivate::syncCustomPalette()
 {
+    bool changed = false;
     for (auto state : { QPalette::Active, QPalette::Inactive, QPalette::Disabled }) {
-        setPaletteColor(customPalette, state, QPalette::WindowText, q->textColor());
-        setPaletteColor(customPalette, state, QPalette::Window, q->backgroundColor());
-        setPaletteColor(customPalette, state, QPalette::Base, q->backgroundColor());
-        setPaletteColor(customPalette, state, QPalette::Text, q->textColor());
-        setPaletteColor(customPalette, state, QPalette::Button, q->backgroundColor());
-        setPaletteColor(customPalette, state, QPalette::ButtonText, q->textColor());
-        setPaletteColor(customPalette, state, QPalette::Highlight, q->highlightColor());
-        setPaletteColor(customPalette, state, QPalette::HighlightedText, q->highlightedTextColor());
-        setPaletteColor(customPalette, state, QPalette::ToolTipBase, q->backgroundColor());
-        setPaletteColor(customPalette, state, QPalette::ToolTipText, q->textColor());
-        setPaletteColor(customPalette, state, QPalette::Link, q->linkColor());
-        setPaletteColor(customPalette, state, QPalette::LinkVisited, q->visitedLinkColor());
+        setPaletteColor(customPalette, state, QPalette::WindowText, q->textColor(), &changed);
+        setPaletteColor(customPalette, state, QPalette::Window, q->backgroundColor(), &changed);
+        setPaletteColor(customPalette, state, QPalette::Base, q->backgroundColor(), &changed);
+        setPaletteColor(customPalette, state, QPalette::Text, q->textColor(), &changed);
+        setPaletteColor(customPalette, state, QPalette::Button, q->backgroundColor(), &changed);
+        setPaletteColor(customPalette, state, QPalette::ButtonText, q->textColor(), &changed);
+        setPaletteColor(customPalette, state, QPalette::Highlight, q->highlightColor(), &changed);
+        setPaletteColor(customPalette, state, QPalette::HighlightedText, q->highlightedTextColor(), &changed);
+        setPaletteColor(customPalette, state, QPalette::ToolTipBase, q->backgroundColor(), &changed);
+        setPaletteColor(customPalette, state, QPalette::ToolTipText, q->textColor(), &changed);
+        setPaletteColor(customPalette, state, QPalette::Link, q->linkColor(), &changed);
+        setPaletteColor(customPalette, state, QPalette::LinkVisited, q->visitedLinkColor(), &changed);
+    }
+    if (changed) {
+        Q_EMIT q->paletteChanged(customPalette);
     }
-    emit q->paletteChanged(customPalette);
 }
 
 void PlatformThemePrivate::findParentStyle()
@@ -161,12 +166,18 @@ void PlatformThemePrivate::findParentStyle()
                 q->setCustomBackgroundColor(t->d->customBackgroundColor);
                 q->setCustomAlternateBackgroundColor(t->d->customAlternateBackgroundColor);
                 q->setCustomHighlightColor(t->d->customHighlightColor);
+                q->setCustomActiveBackgroundColor(t->d->customActiveBackgroundColor);
+                q->setCustomLinkBackgroundColor(t->d->customLinkBackgroundColor);
+                q->setCustomVisitedLinkBackgroundColor(t->d->customVisitedLinkBackgroundColor);
+                q->setCustomNegativeBackgroundColor(t->d->customNegativeBackgroundColor);
+                q->setCustomNeutralBackgroundColor(t->d->customNeutralBackgroundColor);
+                q->setCustomPositiveBackgroundColor(t->d->customPositiveBackgroundColor);
                 q->setCustomFocusColor(t->d->customFocusColor);
                 q->setCustomHoverColor(t->d->customHoverColor);
             }
             break;
         }
-        
+
     }
 }
 
@@ -175,7 +186,7 @@ QColor PlatformThemePrivate::tint(const QColor &c1, const QColor &c2, qreal rati
     qreal r = c1.redF() + (c2.redF() - c1.redF()) * ratio;
     qreal g = c1.greenF() + (c2.greenF() - c1.greenF()) * ratio;
     qreal b = c1.blueF() + (c2.blueF() - c1.blueF()) * ratio;
- 
+
     return QColor::fromRgbF(r, g, b, 1);
 }
 
@@ -188,7 +199,7 @@ void PlatformThemePrivate::emitCompressedColorChanged()
     m_pendingColorChange = true;
     QMetaObject::invokeMethod(q, [this]() {
         syncCustomPalette();
-        emit q->colorsChanged();
+        Q_EMIT q->colorsChanged();
         m_pendingColorChange = false;
     }, Qt::QueuedConnection);
 }
@@ -236,7 +247,7 @@ void PlatformTheme::setColorSet(PlatformTheme::ColorSet colorSet)
     }
 
     if (!d->m_init) {
-        emit colorSetChanged(colorSet);
+        Q_EMIT colorSetChanged(colorSet);
         d->emitCompressedColorChanged();
     }
 }
@@ -261,7 +272,7 @@ void PlatformTheme::setColorGroup(PlatformTheme::ColorGroup colorGroup)
     }
 
     if (!d->m_init) {
-        emit colorGroupChanged(colorGroup);
+        Q_EMIT colorGroupChanged(colorGroup);
         d->emitCompressedColorChanged();
     }
 }
@@ -286,7 +297,7 @@ void PlatformTheme::setInherit(bool inherit)
     if (inherit && d->m_parentTheme) {
         setColorSet(d->m_parentTheme->colorSet());
     }
-    emit inheritChanged(inherit);
+    Q_EMIT inheritChanged(inherit);
 }
 
 QColor PlatformTheme::textColor() const
@@ -324,31 +335,61 @@ QColor PlatformTheme::activeTextColor() const
     return d->customActiveTextColor.isValid() ? d->customActiveTextColor : d->activeTextColor;
 }
 
+QColor PlatformTheme::activeBackgroundColor() const
+{
+    return d->customActiveBackgroundColor.isValid() ? d->customActiveBackgroundColor : d->activeBackgroundColor;
+}
+
 QColor PlatformTheme::linkColor() const
 {
     return d->customLinkColor.isValid() ? d->customLinkColor : d->linkColor;
 }
 
+QColor PlatformTheme::linkBackgroundColor() const
+{
+    return d->customLinkBackgroundColor.isValid() ? d->customLinkBackgroundColor : d->linkBackgroundColor;
+}
+
 QColor PlatformTheme::visitedLinkColor() const
 {
     return d->customVisitedLinkColor.isValid() ? d->customVisitedLinkColor : d->visitedLinkColor;
 }
 
+QColor PlatformTheme::visitedLinkBackgroundColor() const
+{
+    return d->customVisitedLinkBackgroundColor.isValid() ? d->customVisitedLinkBackgroundColor : d->visitedLinkBackgroundColor;
+}
+
 QColor PlatformTheme::negativeTextColor() const
 {
     return d->customNegativeTextColor.isValid() ? d->customNegativeTextColor : d->negativeTextColor;
 }
 
+QColor PlatformTheme::negativeBackgroundColor() const
+{
+    return d->customNegativeBackgroundColor.isValid() ? d->customNegativeBackgroundColor : d->negativeBackgroundColor;
+}
+
 QColor PlatformTheme::neutralTextColor() const
 {
     return d->customNeutralTextColor.isValid() ? d->customNeutralTextColor : d->neutralTextColor;
 }
 
+QColor PlatformTheme::neutralBackgroundColor() const
+{
+    return d->customNeutralBackgroundColor.isValid() ? d->customNeutralBackgroundColor : d->neutralBackgroundColor;
+}
+
 QColor PlatformTheme::positiveTextColor() const
 {
     return d->customPositiveTextColor.isValid() ? d->customPositiveTextColor : d->positiveTextColor;
 }
 
+QColor PlatformTheme::positiveBackgroundColor() const
+{
+    return d->customPositiveBackgroundColor.isValid() ? d->customPositiveBackgroundColor : d->positiveBackgroundColor;
+}
+
 QColor PlatformTheme::focusColor() const
 {
     return d->customFocusColor.isValid() ? d->customFocusColor : d->focusColor;
@@ -430,6 +471,16 @@ void PlatformTheme::setActiveTextColor(const QColor &color)
     d->emitCompressedColorChanged();
 }
 
+void PlatformTheme::setActiveBackgroundColor(const QColor &color)
+{
+    if (d->activeBackgroundColor == color) {
+        return;
+    }
+
+    d->activeBackgroundColor = color;
+    d->emitCompressedColorChanged();
+}
+
 void PlatformTheme::setLinkColor(const QColor &color)
 {
     if (d->linkColor == color) {
@@ -440,6 +491,16 @@ void PlatformTheme::setLinkColor(const QColor &color)
     d->emitCompressedColorChanged();
 }
 
+void PlatformTheme::setLinkBackgroundColor(const QColor &color)
+{
+    if (d->linkBackgroundColor == color) {
+        return;
+    }
+
+    d->linkBackgroundColor = color;
+    d->emitCompressedColorChanged();
+}
+
 void PlatformTheme::setVisitedLinkColor(const QColor &color)
 {
     if (d->visitedLinkColor == color) {
@@ -450,6 +511,16 @@ void PlatformTheme::setVisitedLinkColor(const QColor &color)
     d->emitCompressedColorChanged();
 }
 
+void PlatformTheme::setVisitedLinkBackgroundColor(const QColor &color)
+{
+    if (d->visitedLinkBackgroundColor == color) {
+        return;
+    }
+
+    d->visitedLinkBackgroundColor = color;
+    d->emitCompressedColorChanged();
+}
+
 void PlatformTheme::setNegativeTextColor(const QColor &color)
 {
     if (d->negativeTextColor == color) {
@@ -460,6 +531,16 @@ void PlatformTheme::setNegativeTextColor(const QColor &color)
     d->emitCompressedColorChanged();
 }
 
+void PlatformTheme::setNegativeBackgroundColor(const QColor &color)
+{
+    if (d->negativeBackgroundColor == color) {
+        return;
+    }
+
+    d->negativeBackgroundColor = color;
+    d->emitCompressedColorChanged();
+}
+
 void PlatformTheme::setNeutralTextColor(const QColor &color)
 {
     if (d->neutralTextColor == color) {
@@ -470,6 +551,16 @@ void PlatformTheme::setNeutralTextColor(const QColor &color)
     d->emitCompressedColorChanged();
 }
 
+void PlatformTheme::setNeutralBackgroundColor(const QColor &color)
+{
+    if (d->neutralBackgroundColor == color) {
+        return;
+    }
+
+    d->neutralBackgroundColor = color;
+    d->emitCompressedColorChanged();
+}
+
 void PlatformTheme::setPositiveTextColor(const QColor &color)
 {
     if (d->positiveTextColor == color) {
@@ -480,6 +571,16 @@ void PlatformTheme::setPositiveTextColor(const QColor &color)
     d->emitCompressedColorChanged();
 }
 
+void PlatformTheme::setPositiveBackgroundColor(const QColor &color)
+{
+    if (d->positiveBackgroundColor == color) {
+        return;
+    }
+
+    d->positiveBackgroundColor = color;
+    d->emitCompressedColorChanged();
+}
+
 void PlatformTheme::setHoverColor(const QColor &color)
 {
     if (d->hoverColor == color) {
@@ -512,9 +613,21 @@ void PlatformTheme::setDefaultFont(const QFont &font)
     }
 
     d->font = font;
-    emit defaultFontChanged(font);
+    Q_EMIT defaultFontChanged(font);
 }
 
+QFont PlatformTheme::smallFont() const
+{
+    return d->smallFont;
+}
+
+void PlatformTheme::setSmallFont(const QFont &font)
+{
+    if (d->smallFont != font) {
+        d->smallFont = font;
+        Q_EMIT smallFontChanged(font);
+    }
+}
 
 #define PROPAGATECUSTOMCOLOR(colorName, color)\
             for (PlatformTheme *t : qAsConst(d->m_childThemes)) {\
@@ -602,6 +715,17 @@ void PlatformTheme::setCustomActiveTextColor(const QColor &color)
     d->emitCompressedColorChanged();
 }
 
+void PlatformTheme::setCustomActiveBackgroundColor(const QColor &color)
+{
+    if (d->customActiveBackgroundColor == color) {
+        return;
+    }
+
+    d->customActiveBackgroundColor = color;
+    PROPAGATECUSTOMCOLOR(CustomActiveBackgroundColor, color)
+    d->emitCompressedColorChanged();
+}
+
 void PlatformTheme::setCustomLinkColor(const QColor &color)
 {
     if (d->customLinkColor == color) {
@@ -613,6 +737,17 @@ void PlatformTheme::setCustomLinkColor(const QColor &color)
     d->emitCompressedColorChanged();
 }
 
+void PlatformTheme::setCustomLinkBackgroundColor(const QColor &color)
+{
+    if (d->customLinkBackgroundColor == color) {
+        return;
+    }
+
+    d->customLinkBackgroundColor = color;
+    PROPAGATECUSTOMCOLOR(CustomLinkBackgroundColor, color)
+    d->emitCompressedColorChanged();
+}
+
 void PlatformTheme::setCustomVisitedLinkColor(const QColor &color)
 {
     if (d->customVisitedLinkColor == color) {
@@ -624,6 +759,17 @@ void PlatformTheme::setCustomVisitedLinkColor(const QColor &color)
     d->emitCompressedColorChanged();
 }
 
+void PlatformTheme::setCustomVisitedLinkBackgroundColor(const QColor &color)
+{
+    if (d->customVisitedLinkBackgroundColor == color) {
+        return;
+    }
+
+    d->customVisitedLinkBackgroundColor = color;
+    PROPAGATECUSTOMCOLOR(CustomVisitedLinkBackgroundColor, color)
+    d->emitCompressedColorChanged();
+}
+
 void PlatformTheme::setCustomNegativeTextColor(const QColor &color)
 {
     if (d->customNegativeTextColor == color) {
@@ -635,6 +781,17 @@ void PlatformTheme::setCustomNegativeTextColor(const QColor &color)
     d->emitCompressedColorChanged();
 }
 
+void PlatformTheme::setCustomNegativeBackgroundColor(const QColor &color)
+{
+    if (d->customNegativeBackgroundColor == color) {
+        return;
+    }
+
+    d->customNegativeBackgroundColor = color;
+    PROPAGATECUSTOMCOLOR(CustomNegativeBackgroundColor, color)
+    d->emitCompressedColorChanged();
+}
+
 void PlatformTheme::setCustomNeutralTextColor(const QColor &color)
 {
     if (d->customNeutralTextColor == color) {
@@ -646,6 +803,17 @@ void PlatformTheme::setCustomNeutralTextColor(const QColor &color)
     d->emitCompressedColorChanged();
 }
 
+void PlatformTheme::setCustomNeutralBackgroundColor(const QColor &color)
+{
+    if (d->customNeutralBackgroundColor == color) {
+        return;
+    }
+
+    d->customNeutralBackgroundColor = color;
+    PROPAGATECUSTOMCOLOR(CustomNeutralBackgroundColor, color)
+    d->emitCompressedColorChanged();
+}
+
 void PlatformTheme::setCustomPositiveTextColor(const QColor &color)
 {
     if (d->customPositiveTextColor == color) {
@@ -657,6 +825,17 @@ void PlatformTheme::setCustomPositiveTextColor(const QColor &color)
     d->emitCompressedColorChanged();
 }
 
+void PlatformTheme::setCustomPositiveBackgroundColor(const QColor &color)
+{
+    if (d->customPositiveBackgroundColor == color) {
+        return;
+    }
+
+    d->customPositiveBackgroundColor = color;
+    PROPAGATECUSTOMCOLOR(CustomPositiveBackgroundColor, color)
+    d->emitCompressedColorChanged();
+}
+
 void PlatformTheme::setCustomHoverColor(const QColor &color)
 {
     if (d->customHoverColor == color) {
@@ -694,7 +873,7 @@ void PlatformTheme::setPalette(const QPalette &palette)
 
     d->palette = palette;
     PROPAGATECUSTOMCOLOR(Palette, palette)
-    emit paletteChanged(palette);
+    Q_EMIT paletteChanged(palette);
 }
 
 QIcon PlatformTheme::iconFromTheme(const QString &name, const QColor &customColor)
@@ -724,7 +903,16 @@ PlatformTheme *PlatformTheme::qmlAttachedProperties(QObject *object)
         return PlatformThemePrivate::s_pluginFactory->createPlatformTheme(object);
     } else if (!s_factoryChecked) {
         s_factoryChecked = true;
-#if QT_CONFIG(library)
+
+#ifdef KIRIGAMI_BUILD_TYPE_STATIC
+        for (QObject* staticPlugin : QPluginLoader::staticInstances()) {
+            KirigamiPluginFactory *factory = qobject_cast<KirigamiPluginFactory *>(staticPlugin);
+            if (factory) {
+                PlatformThemePrivate::s_pluginFactory = factory;
+                return factory->createPlatformTheme(object);
+            }
+        }
+#else
         const auto libraryPaths = QCoreApplication::libraryPaths();
         for (const QString &path : libraryPaths) {
             QDir dir(path + QStringLiteral("/kf5/kirigami"));
